@@ -10,7 +10,7 @@ use lib '../XMLCompile/lib', '../../XMLCompile/lib';
 
 package TestTools;
 use vars '$VERSION';
-$VERSION = '0.56';
+$VERSION = '0.57';
 use base 'Exporter';
 
 use XML::LibXML;
@@ -259,8 +259,8 @@ sub test_rw($$$$;$$)
     compare_xml($tree2, $tree->toString);
 }
 
-sub compare_xml($$)
-{   my ($tree, $expect) = @_;
+sub compare_xml($$;$)
+{   my ($tree, $expect, $msg) = @_;
     my $dump = ref $tree ? $tree->toString : $tree;
 
     if(!defined $dump) { ; }
@@ -278,7 +278,7 @@ sub compare_xml($$)
             s/\s+\z//gs;
         }
     }
-    is($dump, $expect);
+    is($dump, $expect, $msg);
 }
 
 sub templ_xml($$$@)
